#!/bin/bash

#from Laurent Excoffier February 2011
# Modified Nina Marchi June 2020
# The script will launch several jobs of fastsimcoal to estimate
# demographic parameters from the SFS, using a each time using a
# conditional maximization (CM) of the parameter likelihood
# This should run on any kind of SFS files generated byfastsimcoal
#
# It assumes the following structure of the observed sfs files:
#	scriptDir
#   	|
#   	|- - - *.est file
#   	|- - - *.tpl file
#   	|- - - *.sh scripts
#   	|- - - fastsimcoal
#   	|- - - targetDir
#   				|
#   				- - - - - subDir for simulated data set 1
#   				|					|
#   				|					|----- *.obs files
#   				|
#   				- - - - - subDir for simulated data set 2
#   				|					|
#   				|					|----- *.obs files
#   				|
#   				- - - - - subDir for simulated data set 3
#   				|					|
#   				|					|----- *.obs files
#   				|
#   				...
#

fsc=fsc36091
msgs=conOutput
mymail=xxx

#if doComputations==0, the program will just collect the results previously generated
#doComputations=0
doComputations=1
queue="all"
#-------- Do computation or collect results ------
if [ $doComputations -eq 1 ]; then
	doCollectResults=0
else
	doCollectResults=1
fi

#-------- Number of different runs per data set ------
numRuns=50
runBase=1 
# jobcount=0
#-----------------------------

#SFStype="-m" ##minor allele freq
SFStype="-d" ##derived allele freq

mkdir $msgs 2>/dev/null

#-------- Default run values ------
iniNumSims=500000 ##-n 
#maxNumSims=200000
minNumLoopsInBrentOptimization=100 ##-l
maxNumLoopsInBrentOptimization=100 ##-L
minValidSFSEntry=5 ##-C
numCores=1 ##-c
numBatches=1 ##-B


#------ Use monomorphic sites or not---------
#useMonoSites="-0" #Uncomment this line NOT to use monomorphic sites
useMonoSites="" #Uncomment this line to use monomorphic sites

#-------- Estimate Ancestral State Misidentification ----------
#asm="--ASM"
asm=""
#----------multiSFS------------
#multiSFS=""
multiSFS="--multiSFS"

#----------logprecision------------
#logprecision=""
logprecision="--logprecision 18"

#-----------------------------
quiet="-q"

#-----------------------------
dircount=0
for case in Model1 Model2
do
	let dircount=dircount+1	
	for pop in Scenario1
	do
		job_description="${case}" #extract 3 first characters of pop
		echo "job_description=$job_description" #debug
			
		#-------- Generic Name ------
		fileDirName=${pop}${case}
		genericName=${pop}${case}
		tplGenericName="${pop}${case}"
			
		popDir="${pop}${case}"
		mkdir $popDir 2>/dev/null
		if [ $doComputations -eq 1 ]; then
			estFile=${tplGenericName}.est
			tplFile=${tplGenericName}.tpl   	
				
			for (( runsDone=$runBase; runsDone<=$numRuns; runsDone++ ))
			do
				runDir="run$runsDone"
				mkdir $fileDirName/$runDir 2>/dev/null
				echo "--------------------------------------------------------------------"
				echo ""
				echo "Current run: $runDir"
				echo ""
				cd $fileDirName/$runDir
				
				#Copying necessary files
				cp ../../${tplFile} .
				cp ../../${estFile} .
				cp ../../${genericName}_DSFS.obs .

				jobName=${pop}${case}_${runsDone}.sh
					
				#Creating bash file on the fly
				(
				echo "#!/bin/bash"
				echo "#Slurm options"
				echo "#SBATCH --workdir=`pwd`"
				echo "#SBATCH --mail-user=${mymail}"
				echo ""
				echo "# specify resources needed"
				echo "#SBATCH --time=500:00:00"
				echo ""
				echo "#SBATCH --output=\"../../$msgs/${pop}${case}_$runsDone.out\""
				echo "#SBATCH --error=\"../../$msgs/${pop}${case}_$runsDone.err\""
				echo "#SBATCH --mail-type=none"
				echo "#SBATCH --partition=$queue"
				echo "#SBATCH --cpus-per-task=$numCores"
				echo ""
				echo "export OMP_NUM_THREADS=$numCores"
				echo ""
				echo "#chmod +x ./$fsc"
				echo ""
				echo "echo \"Analysis of file $genericName in case ${case}\""
				echo "#Computing likelihood of the parameters using the CM-Brent algorithm"
				echo "echo \"\""
		  		echo "../../$fsc -t ${genericName}.tpl -n$iniNumSims $SFStype -e ${genericName}.est -M -l$minNumLoopsInBrentOptimization -L$maxNumLoopsInBrentOptimization $quiet ${useMonoSites} -C${minValidSFSEntry} ${multiSFS} ${logprecision} -c${numCores} -B${numBatches} $asm"                           
				echo "#ls -al > ${genericName}current_directory_listing.txt"
				echo ""
				echo "echo \"\""
				echo "echo \"\""
				echo "echo \"Job $runsDone terminated\""
				) > $jobName
				chmod +x $jobName

				echo "Bash file $jobName created"
				sbatch ./${jobName}
				cd ../.. #$runDir
				done
		fi
		if [ $doCollectResults -eq 1 ]; then
			for (( runsDone=$runBase; runsDone<=$numRuns; runsDone++ ))
			do
				runDir="run$runsDone"
				cd $fileDirName/$runDir
				cd $genericName
				echo "Extracting parameters from run $runsDone"
				#Processing best likelihood files
				bestlhoodFile=${genericName}.bestlhoods
				#Extract second line
				if [ $runsDone -eq 1 ]; 	then
					header=$(sed '1!d'  $bestlhoodFile)
					echo -e "Run\t$header" > ../${genericName}_bestLhoodsParams.txt
				fi
				wantedParameters=$(sed '2!d'  $bestlhoodFile)
				echo -e "$runDir\t$wantedParameters" >> ../../${genericName}_bestLhoodsParams.txt
				cd ..
				cd ../..
			done
		fi
	done
done
